"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigExtends = void 0;
const path_1 = __importDefault(require("path"));
const h3lp_1 = require("h3lp");
const format_1 = require("./model/format");
const cardinal_1 = require("./model/cardinal");
const Yaml = require('js-yaml');
class ConfigExtends {
    constructor() {
        this._excludes = ['_extends', '_completed'];
    }
    apply(source, target, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            let config = {};
            let _source;
            let from;
            const filesInfo = [];
            if (yield h3lp_1.h3lp.fs.isDirectory(source)) {
                _source = source;
                from = options.from;
            }
            else {
                _source = path_1.default.dirname(source);
                from = path_1.default.parse(source).name;
            }
            yield this.loadSourceConfig(config, filesInfo, _source, options);
            config = this.extends(config);
            config = this.getTargetConfig(config, filesInfo, options);
            if (from !== undefined) {
                config = h3lp_1.h3lp.obj.getValue(config, from);
            }
            if (target) {
                yield this.saveTarget(config, filesInfo, target, options);
            }
            return config;
        });
    }
    extends(source, ...args) {
        const _config = Object.assign(source, args);
        for (const k in _config) {
            _config[k] = this.complete(_config, _config[k]);
        }
        for (const k in _config)
            this.removeFlags(_config[k]);
        return _config;
    }
    loadSourceConfig(config, filesInfo, source, options) {
        return __awaiter(this, void 0, void 0, function* () {
            let sources = [source];
            if (options.addSources) {
                sources = sources.concat(options.addSources);
            }
            for (const _source of sources) {
                const sourcePath = path_1.default.resolve(_source);
                if (!h3lp_1.h3lp.fs.exists(sourcePath)) {
                    throw new Error(`${sourcePath} not exists!`);
                }
                if (yield h3lp_1.h3lp.fs.isDirectory(sourcePath)) {
                    const files = yield this.files(sourcePath, '');
                    for (let j = 0; j < files.length; j++) {
                        const fileInfo = {
                            isMainSource: _source === source,
                            isSingleFile: false,
                            filePath: files[j],
                            fullPath: path_1.default.join(sourcePath, files[j]),
                            names: []
                        };
                        yield this.loadSourceConfigFromFile(config, fileInfo);
                        filesInfo.push(fileInfo);
                    }
                }
                else {
                    const fileInfo = {
                        isMainSource: _source === source,
                        isSingleFile: true,
                        filePath: path_1.default.basename(_source),
                        fullPath: sourcePath,
                        names: []
                    };
                    yield this.loadSourceConfigFromFile(config, fileInfo);
                    filesInfo.push(fileInfo);
                }
            }
        });
    }
    loadSourceConfigFromFile(config, fileInfo) {
        return __awaiter(this, void 0, void 0, function* () {
            this.completeFileInfo(fileInfo);
            const content = yield h3lp_1.h3lp.fs.read(fileInfo.fullPath);
            if (content === null) {
                return;
            }
            let data = null;
            if (fileInfo.format === format_1.ExtendsFormat.yaml) {
                const list = yield Yaml.loadAll(content);
                if (list.length === 1) {
                    data = list[0];
                }
                else {
                    fileInfo.cardinal = cardinal_1.ExtendsCardinal.multiple;
                    data = { list };
                }
            }
            else {
                data = JSON.parse(content);
            }
            if (fileInfo.isSingleFile) {
                config = Object.assign(config, data);
            }
            else {
                this.setData(config, fileInfo.names, data);
            }
        });
    }
    saveTarget(config, filesInfo, target, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const targetPath = path_1.default.resolve(target);
            if (yield h3lp_1.h3lp.fs.isDirectory(targetPath)) {
                for (let i = 0; i < filesInfo.length; i++) {
                    const fileInfo = filesInfo[i];
                    const fileData = this.getData(config, fileInfo.names);
                    if (fileData) {
                        fileInfo.targetPath = path_1.default.join(targetPath, fileInfo.filePath);
                        this.writeFile(fileInfo, fileData, options);
                    }
                }
            }
            else {
                const fileInfo = {
                    isMainSource: false,
                    isSingleFile: true,
                    filePath: path_1.default.basename(targetPath),
                    fullPath: targetPath,
                    targetPath,
                    names: []
                };
                this.completeFileInfo(fileInfo);
                this.writeFile(fileInfo, config, options);
            }
        });
    }
    getTargetConfig(config, filesInfo, options) {
        let result = {};
        if (options.outputs) {
            for (const output of options.outputs) {
                const names = output.split('.');
                const data = this.getData(config, names);
                this.setData(result, names, data);
            }
        }
        else {
            if (filesInfo.length === 1 && filesInfo[0].isSingleFile) {
                result = config;
            }
            else {
                for (const fileInfo of filesInfo) {
                    if (fileInfo.isMainSource) {
                        const data = this.getData(config, fileInfo.names);
                        this.setData(result, fileInfo.names, data);
                    }
                }
            }
        }
        return result;
    }
    completeFileInfo(fileInfo) {
        if (!fileInfo.cardinal) {
            fileInfo.cardinal = cardinal_1.ExtendsCardinal.single;
        }
        fileInfo.extension = path_1.default.parse(fileInfo.fullPath).ext.toLocaleLowerCase();
        fileInfo.format = (fileInfo.extension === '.yaml' || fileInfo.extension === '.yml') ? format_1.ExtendsFormat.yaml : format_1.ExtendsFormat.json;
        fileInfo.names = this.names(fileInfo.filePath);
    }
    names(_path) {
        const parsePath = path_1.default.parse(_path);
        const a = parsePath.dir.split(path_1.default.sep);
        const b = parsePath.name.split('.');
        return a.length > 0 && a[0] !== '' ? a.concat(b) : b;
    }
    writeFile(fileInfo, data, options) {
        return __awaiter(this, void 0, void 0, function* () {
            let content;
            const outputFormat = options.format || fileInfo.format;
            if (outputFormat === format_1.ExtendsFormat.yaml) {
                if (fileInfo.cardinal === cardinal_1.ExtendsCardinal.multiple) {
                    content = '';
                    for (let i = 0; i < data.list.length; i++) {
                        const item = data.list[i];
                        if (Object.entries(item).length === 0)
                            continue;
                        content = content + Yaml.dump(item, { noRefs: true });
                        if (i !== (data.list.length - 1))
                            content = content + '---\n';
                    }
                }
                else {
                    content = Yaml.dump(data, { noRefs: true });
                }
            }
            else {
                content = JSON.stringify(data, null, 2);
            }
            if (fileInfo.targetPath === undefined) {
                throw Error('targetPath undefined');
            }
            if (yield h3lp_1.h3lp.fs.exists(path_1.default.dirname(fileInfo.targetPath))) {
                yield h3lp_1.h3lp.fs.create(path_1.default.dirname(fileInfo.targetPath));
            }
            yield h3lp_1.h3lp.fs.write(fileInfo.targetPath, content);
        });
    }
    files(rootPath, relativePath) {
        return __awaiter(this, void 0, void 0, function* () {
            let files = [];
            const list = yield h3lp_1.h3lp.fs.readdir(path_1.default.join(rootPath, relativePath));
            for (const item of list) {
                const fullPath = path_1.default.join(rootPath, relativePath, item);
                if (!(yield h3lp_1.h3lp.fs.isDirectory(fullPath))) {
                    const extension = path_1.default.parse(fullPath).ext.toLocaleLowerCase();
                    if (extension === '.yaml' || extension === '.yml' || extension === '.json') {
                        files.push(path_1.default.join(relativePath, item));
                    }
                }
                else {
                    const children = yield this.files(rootPath, path_1.default.join(relativePath, item));
                    files = files.concat(children);
                }
            }
            return files;
        });
    }
    complete(config, obj, parentParams = {}) {
        if (!obj)
            return obj;
        const params = obj._params ? Object.assign(parentParams, obj._params) : Object.assign(parentParams, {});
        if (obj._extends) {
            const _extends = obj._extends;
            if (Array.isArray(_extends)) {
                for (let i = 0; i < _extends.length; i++) {
                    obj = this.extendObject2(config, obj, _extends[i], params);
                }
            }
            else if (typeof _extends === 'object') {
                for (const k in _extends) {
                    let childParams;
                    if (typeof _extends[k] === 'object') {
                        const newParams = _extends[k];
                        for (const k2 in newParams) {
                            if (typeof newParams[k2] === 'string') {
                                newParams[k2] = this.solveParams(newParams[k2], params);
                            }
                        }
                        childParams = Object.assign(params, newParams);
                    }
                    else {
                        childParams = params;
                    }
                    obj = this.extendObject2(config, obj, k, childParams);
                }
            }
            else {
                obj = this.extendObject2(config, obj, _extends, params);
            }
        }
        for (const k in obj) {
            if (typeof obj[k] === 'object') {
                obj[k] = this.complete(config, obj[k], params);
            }
            if (typeof obj[k] === 'string') {
                obj[k] = this.solveParams(obj[k], params);
            }
        }
        if (obj)
            obj._completed = true;
        return obj;
    }
    solveParams(expression, params) {
        let value = expression;
        if (typeof value !== 'string' || !value.includes('${'))
            return value;
        for (const k in params) {
            if (value.includes('${' + k + '}')) {
                value = h3lp_1.h3lp.str.replace(value, '${' + k + '}', params[k]);
            }
        }
        return isNaN(value) ? value : parseFloat(value);
    }
    extendObject2(config, obj, baseFullName, params) {
        // TODO: mejorar para que no llegue aca este error
        if (this._excludes.includes(baseFullName))
            return obj;
        let base = this.getData(config, baseFullName.split('.'));
        if (base === undefined) {
            throw new Error(`${baseFullName} not found`);
        }
        if (base && !base._completed) {
            base = this.complete(config, base, params);
        }
        return h3lp_1.h3lp.obj.extends(obj, base, this._excludes);
    }
    setData(config, names, data) {
        let _config = config;
        for (let i = 0; i < names.length; i++) {
            if (!_config[names[i]])
                _config[names[i]] = (names.length - 1) === i ? data : {};
            _config = _config[names[i]];
        }
    }
    getData(config, names) {
        let _config = config;
        for (let i = 0; i < names.length; i++) {
            if (!_config)
                break;
            _config = _config[names[i]];
        }
        return _config;
    }
    removeFlags(obj) {
        if (!obj)
            return;
        if (obj._extends)
            delete obj._extends;
        if (obj._params)
            delete obj._params;
        if (obj._completed)
            delete obj._completed;
        for (const k in obj) {
            if (typeof obj[k] === 'object') {
                this.removeFlags(obj[k]);
            }
        }
    }
}
exports.ConfigExtends = ConfigExtends;
//# sourceMappingURL=configExtends.js.map